
# the library path of this plugin
lib_path = File.expand_path( 'lib', bundle_path )

# The ClientPkgCache class:
if RSence.args[:debug]
  load File.expand_path( 'client_pkg_cache.rb', lib_path )
else
  require File.expand_path( 'client_pkg_cache', lib_path )
end

# The ClientPkgServe module:
if RSence.args[:debug]
  load File.expand_path( 'client_pkg_serve.rb', lib_path )
else
  require File.expand_path( 'client_pkg_serve', lib_path )
end

# The ClientPkgBuild class:
if RSence.args[:debug]
  load File.expand_path( 'client_pkg_build.rb', lib_path )
else
  require File.expand_path( 'client_pkg_build', lib_path )
end

# @private The ClientPkgPlugin builds and serves the client packages. It's not intended to be a part of the public API at this time.
class ClientPkgPlugin < Servlet

  include ClientPkgServe

  class BuildLogger
    def initialize( log_path )
      @last_time = Time.now
      @log_path = log_path
      @log_file = nil
    end
    def log( str )
      if RSence.args[:verbose] or not RSence.args[:suppress_build_messages]
        puts str
        return
      else
        if @last_time < Time.now - 30
          @last_time = Time.now
          @log_file.write( %{--- #{@last_time.strftime("%Y-%m-%d %H:%M:%S")} ---\n} )
        end
        @log_file.write( "#{str}\n" )
        @log_file.flush
      end
    end
    def open
      return if @log_file
      @log_file = File.open( @log_path, 'a' )
    end
    def close
      @log_file.close
      @log_file = nil
    end
  end

  def rebuild_client
    @build_busy = true
    next_change=Time.now.to_f
    @client_build.setup_dirs
    @client_build.run( next_change )
    client_cache = ClientPkgCache.new
    client_cache.set_cache( @client_build.js, @client_build.gz, @client_build.themes )
    @client_cache = client_cache
    RSence.plugin_manager.incr! if RSence.config[:transporter_conf][:client_autoreload]
    @last_change = next_change
    @build_busy = false
  end

  def ready?
    return (not @build_busy)
  end

  def open
    if not @thr and RSence.args[:autoupdate]
      @thr = Thread.new do
        Thread.pass
        while true
          begin
            if not @build_busy and @client_build.bundle_changes( @last_change )
              rebuild_client
              puts "Autobuilt."
              if RSence.args[:say]
                Thread.new do
                  Thread.pass
                  system('say "Autobuilt."')
                end
              end
            end
          rescue => e
            warn e.inspect
          end
          sleep 3
        end
      end
    end

  end

  def add_src_dir( src_dir ); @client_build.add_src_dir( src_dir ); end
  def add_src_dirs( src_dirs ); @client_build.add_src_dirs( src_dirs ); end
  def del_src_dir( src_dir ); @client_build.del_src_dir( src_dir ); end
  def del_src_dirs( src_dirs ); @client_build.del_src_dirs( src_dirs ); end

  def add_theme( theme_name ); @client_build.add_theme( theme_name ); end
  def add_themes( theme_names ); @client_build.add_themes( theme_names ); end
  def del_theme( theme_name ); @client_build.del_theme( theme_name ); end
  def del_themes( theme_names ); @client_build.del_themes( theme_names ); end

  def add_package( pkg_name, pkg_items ); @client_build.add_package( pkg_name, pkg_items ); end
  def add_packages( packages ); @client_build.add_packages( packages ); end
  def del_package( pkg_name ); @client_build.del_package( pkg_name ); end
  def del_packages( packages ); @client_build.del_packages( packages ); end

  def add_compound( compound_name, pkg_names ); @client_build.add_compound( compound_name, pkg_names ); end
  def add_compounds( compounds ); @client_build.add_compounds( compounds ); end
  def del_compound( compound_name ); @client_build.del_compound( compound_name ); end
  def del_compounds( compounds ); @client_build.del_compounds( compounds ); end

  def add_reserved_name( reserved_name ); @client_build.add_reserved_name( reserved_name ); end
  def add_reserved_names( reserved_names ); @client_build.add_reserved_names( reserved_names ); end
  def del_reserved_name( reserved_name ); @client_build.del_reserved_name( reserved_name ); end
  def del_reserved_names( reserved_names ); @client_build.del_reserved_names( reserved_names ); end

  def add_gfx_format( gfx_format_name ); @client_build.add_gfx_format( gfx_format_name ); end
  def add_gfx_formats( gfx_format_names ); @client_build.add_gfx_formats( gfx_format_names ); end
  def del_gfx_format( gfx_format_name ); @client_build.del_gfx_format( gfx_format_name ); end
  def del_gfx_formats( gfx_format_names ); @client_build.del_gfx_formats( gfx_format_names ); end

  def client_build; @client_build; end

  def client_cache; @client_cache; end

  def close
    if @thr
      @thr.kill
      @thr = false
    end
    @client_build.flush
    @client_build = nil
    @client_cache = nil
    @build_logger.close
  end

  def squeeze( js )
    return @client_build.squeeze( js )
  end

  def coffee( src )
    return @client_build.coffee( src )
  end

  def init

    @thr = false

    @build_logger = BuildLogger.new( File.join(RSence.args[:env_path],'log','build_log') )
    @build_logger.open

    @client_build = ClientPkgBuild.new( RSence.config[:client_pkg], @build_logger )
    @client_cache = ClientPkgCache.new

    @build_busy = false
    rebuild_client

  end

end


